function createGraph(
  svgId,
  pointsGroupId,
  xCoords,
  yCoords,
  imageUrls,
  captions
) {
  const svg = document.getElementById(svgId);
  const pointsGroup = document.getElementById(pointsGroupId);

  // adjust viewBox to accommodate labels outside the graph
  svg.setAttribute("viewBox", "0 0 360 260");

  // helper functions
  function roundToNearest(value, nearest) {
    return Math.round(value / nearest) * nearest;
  }

  function getMinMaxRounded(values, nearest) {
    const min = Math.min(...values);
    const max = Math.max(...values);
    return {
      min: Math.max(0, roundToNearest(min, nearest) - nearest),
      max: roundToNearest(max, nearest) + nearest,
    };
  }

  // calculate x and y ranges
  const xRange = getMinMaxRounded(xCoords, 40);
  const yRange = getMinMaxRounded(yCoords, 0.2);

  // calculate scales
  const xScale = 280 / (xRange.max - xRange.min);
  const yScale = 180 / (yRange.max - yRange.min);

  // add graph border
  const border = document.createElementNS("http://www.w3.org/2000/svg", "rect");
  border.setAttribute("x", "60");
  border.setAttribute("y", "20");
  border.setAttribute("width", "280");
  border.setAttribute("height", "180");
  border.setAttribute("fill", "none");
  border.setAttribute("stroke", "white");
  svg.insertBefore(border, pointsGroup);

  // add X-axis label
  const xLabel = document.createElementNS("http://www.w3.org/2000/svg", "text");
  xLabel.setAttribute("x", "200");
  xLabel.setAttribute("y", "245");
  xLabel.setAttribute("text-anchor", "middle");
  xLabel.setAttribute("fill", "#4f9b9e");
  xLabel.setAttribute("font-size", "0.8vw");
  xLabel.setAttribute("font-weight", "bold");
  xLabel.textContent = "Time (hr)";
  svg.appendChild(xLabel);

  // add Y-axis label
  const yLabel = document.createElementNS("http://www.w3.org/2000/svg", "text");
  yLabel.setAttribute("x", "25");
  yLabel.setAttribute("y", "110");
  yLabel.setAttribute("text-anchor", "middle");
  yLabel.setAttribute("transform", "rotate(-90, 25, 110)");
  yLabel.setAttribute("fill", "#4f9b9e");
  yLabel.setAttribute("font-size", "0.8vw");
  yLabel.setAttribute("font-weight", "bold");
  yLabel.textContent = "OD 730 (nm)";
  svg.appendChild(yLabel);

  // add X-axis ticks and values
  const xTicks = Math.ceil((xRange.max - xRange.min) / 40) + 1;
  for (let i = 0; i < xTicks; i++) {
    const xValue = xRange.min + i * 40;
    const x = (xValue - xRange.min) * xScale + 60;
    const tick = document.createElementNS("http://www.w3.org/2000/svg", "line");
    tick.setAttribute("x1", x);
    tick.setAttribute("y1", "200");
    tick.setAttribute("x2", x);
    tick.setAttribute("y2", "205");
    tick.setAttribute("stroke", "white");
    svg.appendChild(tick);

    const value = document.createElementNS(
      "http://www.w3.org/2000/svg",
      "text"
    );
    value.setAttribute("x", x);
    value.setAttribute("y", "220");
    value.setAttribute("text-anchor", "middle");
    value.setAttribute("font-size", "10");
    value.setAttribute("fill", "white");
    value.textContent = xValue.toFixed(0);
    svg.appendChild(value);
  }

  // add Y-axis ticks and values
  const yTicks = Math.ceil((yRange.max - yRange.min) / 0.2) + 1;
  for (let i = 0; i < yTicks; i++) {
    const yValue = yRange.min + i * 0.2;
    const y = 200 - (yValue - yRange.min) * yScale;
    const tick = document.createElementNS("http://www.w3.org/2000/svg", "line");
    tick.setAttribute("x1", "55");
    tick.setAttribute("y1", y);
    tick.setAttribute("x2", "60");
    tick.setAttribute("y2", y);
    tick.setAttribute("stroke", "white");
    svg.appendChild(tick);

    const value = document.createElementNS(
      "http://www.w3.org/2000/svg",
      "text"
    );
    value.setAttribute("x", "50");
    value.setAttribute("y", y + 5);
    value.setAttribute("text-anchor", "end");
    value.setAttribute("font-size", "10");
    value.setAttribute("fill", "white");
    value.textContent = yValue.toFixed(1);
    svg.appendChild(value);
  }

  // add points
  xCoords.forEach((x, index) => {
    const y = yCoords[index];
    const circle = document.createElementNS(
      "http://www.w3.org/2000/svg",
      "circle"
    );

    circle.setAttribute("cx", x * xScale + 60);
    circle.setAttribute("cy", 200 - y * yScale);
    circle.setAttribute("r", 5);
    circle.setAttribute("fill", "#4adee3");

    circle.addEventListener("click", (event) => {
      showImagePopup(event, index, imageUrls, captions);
    });

    pointsGroup.appendChild(circle);
  });

  // create a path to connect the points
  const linePath = document.createElementNS(
    "http://www.w3.org/2000/svg",
    "path"
  );
  let pathData = `M${(xCoords[0] - xRange.min) * xScale + 60} ${
    200 - (yCoords[0] - yRange.min) * yScale
  }`;

  for (let i = 1; i < xCoords.length; i++) {
    pathData += ` L${(xCoords[i] - xRange.min) * xScale + 60} ${
      200 - (yCoords[i] - yRange.min) * yScale
    }`;
  }

  linePath.setAttribute("d", pathData);
  linePath.setAttribute("fill", "none");
  linePath.setAttribute("stroke", "#7bc6b4");
  linePath.setAttribute("stroke-width", "2");

  svg.insertBefore(linePath, pointsGroup);
}

// popup element
const popup = document.createElement("div");
popup.style.position = "fixed";
popup.style.display = "none";
popup.style.background = "white";
popup.style.border = "1px solid black";
popup.style.borderRadius = "10px";
popup.style.padding = "10px";
popup.style.zIndex = "1000";
popup.style.boxShadow = "0 4px 6px rgba(0, 0, 0, 0.1)";
popup.style.overflow = "hidden";
document.body.appendChild(popup);

const popupImage = document.createElement("img");
popupImage.style.width = "300px";
popupImage.style.height = "300px";
popupImage.style.objectFit = "cover";
popupImage.style.transition = "transform 2s ease";
popup.appendChild(popupImage);

const captionDiv = document.createElement("div");
captionDiv.style.textAlign = "center";
captionDiv.style.marginTop = "10px";
captionDiv.style.background = "white";
popup.appendChild(captionDiv);

const closeButton = document.createElement("button");
closeButton.textContent = "Close";
closeButton.style.display = "block";
closeButton.style.marginTop = "10px";
closeButton.addEventListener("click", hideImagePopup);
popup.appendChild(closeButton);

function showImagePopup(event, index, imageUrls, captions) {
  const currentImageUrl = imageUrls[index * 2];
  const nextImageUrl = imageUrls[index * 2 + 1];
  const currentCaption = captions[index * 2];
  const nextCaption = captions[index * 2 + 1];

  if (!currentImageUrl || currentImageUrl === "") {
    popupImage.style.display = "none";
    captionDiv.style.display = "block";
    captionDiv.textContent = currentCaption;
    popup.style.left = `${event.clientX + 10}px`;
    popup.style.top = `${event.clientY + 10}px`;
    popup.style.display = "block";

    closeButton.addEventListener("click", hideImagePopup);
  } else {
    popupImage.style.display = "block";
    captionDiv.style.display = "block";
    popupImage.src = currentImageUrl;
    popupImage.style.transform = "scale(1)";
    popupImage.style.zIndex = "1";
    popupImage.style.objectFit = "cover";

    captionDiv.textContent = currentCaption;
    captionDiv.style.zIndex = "2";

    popup.style.left = `${event.clientX + 10}px`;
    popup.style.top = `${event.clientY + 10}px`;
    popup.style.display = "block";

    popupImage.style.transition = "transform 0.5s ease-in-out";
    popupImage.style.transformOrigin = "50% 50%";

    popupImage.onmouseenter = () => {
      popupImage.style.transform = "scale(3)";
      captionDiv.style.zIndex = "1001";

      setTimeout(() => {
        if (nextImageUrl) {
          popupImage.src = nextImageUrl;
          captionDiv.textContent = nextCaption;
        }
      }, 500);
    };

    popupImage.onmouseleave = () => {
      popupImage.style.transform = "scale(1)";
      setTimeout(() => {
        if (currentImageUrl) {
          popupImage.src = currentImageUrl;
          captionDiv.textContent = currentCaption;
        }
      }, 500);
    };
  }
}

function hideImagePopup() {
  popup.style.display = "none";
  popupImage.style.transform = "scale(1)";
  popupImage.onmouseenter = null;
  popupImage.onmouseleave = null;
  popupImage.style.display = "block"; // Reset image display
  captionDiv.style.display = "block"; // Reset caption display
}

// Close popup when clicking outside
document.addEventListener("click", (event) => {
  if (!popup.contains(event.target) && event.target.tagName !== "circle") {
    hideImagePopup();
  }
});

// close popup when clicking outside
document.addEventListener("click", (event) => {
  if (!popup.contains(event.target) && event.target.tagName !== "circle") {
    hideImagePopup();
  }
});

// create three graphs with different data
const xCoords1 = [
  10.96666667, 45.83333333, 69.91666667, 93.76666667, 117.7666667, 141.9833333,
  166.15, 189.9333333, 213.2, 238.5666667, 262.7833333, 336.05,
];
const yCoords1 = [
  0.077, 0.116, 0.229, 0.378, 0.456, 0.713, 0.73, 0.811, 0.861, 0.95, 1.003,
  1.004,
];
const imageUrls1 = [
  "",
  "",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9350.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9351.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9423.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9424.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9482.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9483.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9515.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9516.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9551.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9552.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9594.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9595.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9626.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9627.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9661.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9662.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9697.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9698.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9727.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9728.webp",
  "",
];
const captions1 = [
  "Sep 16th, 2024 | OD: 0.077",
  " ",
  "Sep 17th, 2024 | OD: 0.116",
  " ",
  "Sep 18th, 2024 | OD: 0.229",
  " ",
  "Sep 19th, 2024 | OD: 0.378",
  " ",
  "Sep 20th, 2024 | OD: 0.456",
  " ",
  "Sep 21st, 2024 | OD: 0.713",
  " ",
  "Sep 22nd, 2024 | OD: 0.730",
  " ",
  "Sep 23rd, 2024 | OD: 0.811",
  " ",
  "Sep 24th, 2024 | OD: 0.861",
  " ",
  "Sep 25th, 2024 | OD: 0.950",
  " ",
  "Sep 26th, 2024 | OD: 1.003",
  " ",
  "Sep 30th, 2024 | OD: 1.004",
  " ",
];

const xCoords2 = [
  10.9833, 45.8333, 69.93333333, 93.73333333, 117.7833333, 134.95, 166.15,
  189.91, 213.2, 238.55, 262.7833333,
];
const yCoords2 = [
  0.052, 0.053, 0.136, 0.21, 0.279, 0.433, 0.597, 0.743, 0.768, 0.863, 0.855,
];
const imageUrls2 = [
  "",
  "",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9346.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9347.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9425.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9426.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9484.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9485.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9513.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9514.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9553.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9554.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9592.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9593.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9628.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9629.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9663.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9664.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9695.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9696.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9725.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/sid-sylvia-and-flash/img-9726.webp",
];

const captions2 = [
  "Sep 16th, 2024 | OD: 0.052",
  " ",
  "Sep 17th, 2024 | OD: 0.053",
  " ",
  "Sep 18th, 2024 | OD: 0.136",
  " ",
  "Sep 19th, 2024 | OD: 0.210",
  " ",
  "Sep 20th, 2024 | OD: 0.279",
  " ",
  "Sep 21st, 2024 | OD: 0.433",
  " ",
  "Sep 22nd, 2024 | OD: 0.597",
  " ",
  "Sep 23rd, 2024 | OD: 0.743",
  " ",
  "Sep 24th, 2024 | OD: 0.768",
  " ",
  "Sep 25th, 2024 | OD: 0.863",
  " ",
  "Sep 26th, 2024 | OD: 0.855",
  " ",
];

const xCoords3 = [
  10, 106.1333333, 136.8833333, 159.9333333, 183.3666667, 207.4333333, 225.35,
  249.8333333, 279.3666667, 303.3166667,
];
const yCoords3 = [
  0.001, 0.06, 0.107, 0.136, 0.186, 0.28, 0.333, 0.475, 0.755, 1.143,
];
const imageUrls3 = [
  "",
  "",
  "",
  "",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/froakie/img-9003.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/froakie/img-9004.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/froakie/img-9083.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/froakie/img-9084.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/froakie/img-9122.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/froakie/img-9123.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/froakie/img-9186.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/froakie/img-9187.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/froakie/img-9252.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/froakie/img-9253.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/froakie/img-9326.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/froakie/img-9327.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/froakie/img-9431.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/froakie/img-9432.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/froakie/img-9476.webp",
  "https://static.igem.wiki/teams/5149/cyano-growth-images/froakie/img-9477.webp",
];
const captions3 = [
  /* captions for graph 3 */
  "Sep 5th, 2024 | OD: 0.001",
  " ",
  "Sep 9th, 2024 | OD: 0.060",
  " ",
  "Sep 10th, 2024 | OD: 0.107",
  " ",
  "Sep 11th, 2024 | OD: 0.136",
  " ",
  "Sep 12th, 2024 | OD: 0.186",
  " ",
  "Sep 13st, 2024 | OD: 0.280",
  " ",
  "Sep 14nd, 2024 | OD: 0.333",
  " ",
  "Sep 15rd, 2024 | OD: 0.475",
  " ",
  "Sep 16th, 2024 | OD: 0.755",
  " ",
  "Sep 17th, 2024 | OD: 1.143",
  " ",
];

createGraph("graph1", "points1", xCoords1, yCoords1, imageUrls1, captions1);
createGraph("graph2", "points2", xCoords2, yCoords2, imageUrls2, captions2);
createGraph("graph3", "points3", xCoords3, yCoords3, imageUrls3, captions3);
